@@ -1,6 +1,6 @@
 /*
- * $Id: maildirquota.c,v 1.17 2007/10/01 06:59:55 rwidmer Exp $
- * Copyright (C) 1999-2004 Inter7 Internet Technologies, Inc.
+ * $Id: maildirquota.c 1029 2011-02-28 16:59:25Z volz0r $
+ * Copyright (C) 1999-2009 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,30 +35,30 @@
 #include "vlimits.h"
 #include "maildirquota.h"
 #include "config.h"
+#include "conf.h"
+#include "storage.h"
+#include "client.h"
 
 /* private functions - no name clashes with courier */
 static char *makenewmaildirsizename(const char *, int *);
-static int countcurnew(const char *, time_t *, off_t *, int *);
-static int countsubdir(const char *, const char *,
-		time_t *, off_t *, int *);
-static int statcurnew(const char *, time_t *);
-static int statsubdir(const char *, const char *, time_t *);
-static int doaddquota(const char *, int, const char *, long, int, int);
-static int docheckquota(const char *dir, int *maildirsize_fdptr,
-	const char *quota_type, long xtra_size, int xtra_cnt, int *percentage);
-static int docount(const char *, time_t *, off_t *, int *);
-static int maildir_checkquota(const char *dir, int *maildirsize_fdptr,
-	const char *quota_type, long xtra_size, int xtra_cnt);
-/*
+static int countcurnew(const char *dir, time_t *maxtime, storage_t *sizep, storage_t *cntp);
+static int countsubdir(const char *dir, const char *subdir, time_t *maxtime, storage_t *sizep, storage_t *cntp);
+static int statcurnew(const char *dir, time_t *maxtimestamp);
+static int statsubdir(const char *dir, const char *subdir, time_t *maxtime);
+static int doaddquota(const char *dir, int maildirsize_fd, const char *quota_type, 
+                      storage_t maildirsize_size, storage_t maildirsize_cnt, int isnew);
+static int docheckquota(const char *dir, int *maildirsize_fdptr, const char *quota_type, 
+                        storage_t xtra_size, storage_t xtra_cnt, int *percentage);
+static int docount(const char *dir, time_t *dirstamp, storage_t *sizep, storage_t *cntp);
+static int maildir_checkquota(const char *dir, int *maildirsize_fdptr, const char *quota_type, storage_t xtra_size, storage_t xtra_cnt);
+/*  moved into maildirquota.h as non-static
 static int maildir_addquota(const char *dir, int maildirsize_fd,
 	const char *quota_type, long maildirsize_size, int maildirsize_cnt);
 */
-
 static int maildir_safeopen(const char *path, int mode, int perm);
 static char *str_pid_t(pid_t t, char *arg);
 static char *str_time_t(time_t t, char *arg);
-static int maildir_parsequota(const char *n, unsigned long *s);
-
+static int maildir_parsequota(const char *n, storage_t *s);
 
 #define  NUMBUFSIZE      60
 #define	MDQUOTA_SIZE	'S'	/* Total size of all messages in maildir */
@@ -66,6 +66,8 @@ static int maildir_parsequota(const char *n, unsigned long *s);
 				maildir -- NOT IMPLEMENTED */
 #define	MDQUOTA_COUNT	'C'	/* Total number of messages in maildir */
 
+int quota_mtos(const char *, storage_t *, storage_t *);
+int quota_user_usage(const char *, storage_t *, storage_t *);
 
 /* bk: add domain limits functionality */
 int domain_over_maildirquota(const char *userdir)
@@ -73,12 +75,14 @@ int domain_over_maildirquota(const char *userdir)
 struct  stat    stat_buf;
 char	domdir[MAX_PW_DIR];
 char	*p;
-char	domain[256];
-long    size = 0;
-unsigned long maxsize = 0;
-int	cnt = 0;
-int	maxcnt = 0;
+char	domain[256], qb[256] = { 0 };
+storage_t size = 0;
+storage_t maxsize = 0;
+storage_t cnt = 0;
+int	ret = 0;
+storage_t maxcnt = 0;
 struct vlimits limits;
+   storage_t susage = 0, cusage = 0;
 
         if (fstat(0, &stat_buf) == 0 && S_ISREG(stat_buf.st_mode) &&
                 stat_buf.st_size > 0)
@@ -99,11 +103,34 @@ struct vlimits limits;
 		maxsize = limits.diskquota * 1024 * 1024;
 		maxcnt = limits.maxmsgcount;
 
+                /* only check the quota if one is set  */
+                if(( maxsize==0 ) && (maxcnt==0)) return 0;
+
 		if (vget_assign (domain, domdir, sizeof(domdir), NULL, NULL) == NULL)
 			return -1;
 
 		/* get the domain usage */
-		if (readdomainquota(domdir, &size, &cnt)) return -1;
+
+		 ret = strlen(domain);
+
+		 if ((ret + 2) < sizeof(qb)) {
+			*qb = '@';
+			memcpy((qb + 1), domain, ret);
+			*(qb + ret + 1) = '\0';
+
+			ret = client_query_quick(qb, &susage, &cusage);
+			if (ret) {
+			   if ((susage + stat_buf.st_size) > maxsize)
+				  return 1;
+
+			   if ((maxcnt) && (cusage >= maxcnt))
+				  return 1;
+
+			   return 0;
+			}
+		 }
+
+		 if (readdomainquota(domdir, &size, &cnt)) return -1;
 
 		/* check if either quota (size/count) would be exceeded */
 		if (maxsize > 0 && (size + stat_buf.st_size) > maxsize) return 1;
@@ -113,7 +140,7 @@ struct vlimits limits;
         return 0;
 }
 
-int readdomainquota(const char *dir, long *sizep, int *cntp)
+int readdomainquota(const char *dir, storage_t *sizep, storage_t *cntp)
 {
 int tries;
 char	checkdir[256];
@@ -169,7 +196,7 @@ struct dirent *de;
 	return 0;
 }
 
-int wrapreaduserquota(const char* dir, off_t *sizep, int *cntp)
+int wrapreaduserquota(const char* dir, storage_t *sizep, storage_t *cntp)
 {
 time_t	tm;
 time_t	maxtime;
@@ -242,10 +269,10 @@ struct dirent *de;
 
 	return 0;
 }
-int readuserquota(const char* dir, long *sizep, int *cntp)
+int readuserquota(const char* dir, storage_t *sizep, storage_t *cntp)
 {
 	int retval;
-	off_t s;
+	storage_t s;
 	
 	s = (off_t) *sizep;
 	retval = wrapreaduserquota(dir, &s, cntp);
@@ -256,19 +283,18 @@ int readuserquota(const char* dir, long *sizep, int *cntp)
 int user_over_maildirquota( const char *dir, const char *q)
 {
 struct  stat    stat_buf;
-int     quotafd;
-int     ret_value;
+int     quotafd = -1;
+int     ret_value = 0;
 
-        if (fstat(0, &stat_buf) == 0 && S_ISREG(stat_buf.st_mode) &&
-                stat_buf.st_size > 0 && *q)
-        {
-                if (maildir_checkquota(dir, &quotafd, q, stat_buf.st_size, 1)
-                        && errno != EAGAIN)
+        //   stat file              is regular file                size > 0               what is q?
+        if (fstat(0, &stat_buf) == 0 && S_ISREG(stat_buf.st_mode) && stat_buf.st_size > 0 && *q)
+        {       //   check the quota, and not again error
+                if (maildir_checkquota(dir, &quotafd, q, stat_buf.st_size, 1) && errno != EAGAIN)
                 {
                         if (quotafd >= 0)       close(quotafd);
                         ret_value = 1;
                 } else {
-                        maildir_addquota(dir, quotafd, q, stat_buf.st_size, 1);
+                        //maildir_addquota(dir, quotafd, q, stat_buf.st_size, 1);
                         if (quotafd >= 0)       close(quotafd);
                         ret_value = 0;
                 }
@@ -302,8 +328,8 @@ char    quotawarnmsg[500];
 
 static int maildirsize_read(const char *filename,	/* The filename */
 	int *fdptr,	/* Keep the file descriptor open */
-	off_t *sizeptr,	/* Grand total of maildir size */
-	int *cntptr, /* Grand total of message count */
+	storage_t *sizeptr,	/* Grand total of maildir size */
+	storage_t *cntptr, /* Grand total of message count */
 	unsigned *nlines, /* # of lines in maildirsize */
 	struct stat *statptr)	/* The stats on maildirsize */
 {
@@ -311,14 +337,27 @@ static int maildirsize_read(const char *filename,	/* The filename */
  int f;
  char *p;
  unsigned l;
- int n;
+ storage_t n;
  int first;
+ int ret = 0;
 
 	if ((f=maildir_safeopen(filename, O_RDWR|O_APPEND, 0)) < 0)
 		return (-1);
 	p=buf;
 	l=sizeof(buf);
 
+	/*
+		 Maildir++ specification says to rebuild the maildirsize file if the
+		 file is 5120 or more bytes, or is more than 15 minutes old
+    */
+
+	ret = fstat(f, statptr);
+	if ((ret != -1) && ((statptr->st_size >= 5120) || (time(NULL) > statptr->st_mtime + (15*60)))) {
+	   unlink(filename);
+	   close(f);
+	   return -1;
+    }
+
 	while (l)
 	{
 		n=read(f, p, l);
@@ -331,7 +370,7 @@ static int maildirsize_read(const char *filename,	/* The filename */
 		p += n;
 		l -= n;
 	}
-	if (l == 0 || fstat(f, statptr))	/* maildir too big */
+	if (l == 0 || ret)	/* maildir too big */
 	{
 		close(f);
 		return (-1);
@@ -345,8 +384,8 @@ static int maildirsize_read(const char *filename,	/* The filename */
 	first=1;
 	while (*p)
 	{
-	long n=0;
-	int c=0;
+	storage_t n=0;
+	storage_t c=0;
 	char	*q=p;
 
 		while (*p)
@@ -361,7 +400,7 @@ static int maildirsize_read(const char *filename,	/* The filename */
 			first=0;
 			continue;
 		}
-		sscanf(q, "%ld %d", &n, &c);
+		sscanf(q, "%llu %llu", &n, &c);
 		*sizeptr += n;
 		*cntptr += c;
 		++ *nlines;
@@ -370,9 +409,9 @@ static int maildirsize_read(const char *filename,	/* The filename */
 	return (0);
 }
 
-static int qcalc(off_t s, unsigned n, const char *quota, int *percentage)
+static int qcalc(storage_t s, storage_t n, const char *quota, int *percentage)
 {
-off_t i;
+storage_t i;
 int	spercentage=0;
 int	npercentage=0;
 
@@ -397,6 +436,7 @@ int	npercentage=0;
 				return (-1);
 			}
 
+#if 0
 			/*
 			** For huge quotas, over 20mb,
 			** divide numerator & denominator by 1024 to prevent
@@ -405,6 +445,7 @@ int	npercentage=0;
 
 			x=1;
 			if (i > 20000000) x=1024;
+#endif
 
 			spercentage = i ? (s/x) * 100 / (i/x):100;
 			break;
@@ -416,10 +457,12 @@ int	npercentage=0;
 				return (-1);
 			}
 
+#if 0
 			/* Ditto */
 
 			x=1;
 			if (i > 20000000) x=1024;
+#endif
 
 			npercentage = i ? ((off_t)n/x) * 100 / (i/x):100;
 			break;
@@ -429,14 +472,33 @@ int	npercentage=0;
 	return (0);
 }
 
-
 static int maildir_checkquota(const char *dir,
 	int *maildirsize_fdptr,
 	const char *quota_type,
-	long xtra_size,
-	int xtra_cnt)
+	storage_t xtra_size,
+	storage_t xtra_cnt)
 {
-int	dummy;
+int	dummy, ret = 0;
+
+   if ((xtra_size == 0) || (xtra_cnt == 0))
+	  return 0;
+
+   /*
+	  Ping the daemon
+   */
+
+   ret = client_query_quick(" ", NULL, NULL);
+   if (ret) {
+      ret = vmaildir_readquota(dir, quota_type);
+	  if (ret >= 100)
+		 return -1;
+
+	  return 0;
+   }
+
+   /*
+	  Fall back
+   */
 
 	return (docheckquota(dir, maildirsize_fdptr, quota_type,
 		xtra_size, xtra_cnt, &dummy));
@@ -444,8 +506,46 @@ int	dummy;
 
 int vmaildir_readquota(const char *dir, const char *quota_type)
 {
-int	percentage=0;
-int	fd=-1;
+   int	percentage=0;
+   int	fd=-1;
+   int ret = 0;
+   char *email = NULL;
+   storage_t uusage = 0, dusage = 0, usquota = 0, ucquota = 0;
+
+   /*
+	  Get user usage
+   */
+
+   email = maildir_to_email(dir);
+   ret = client_query_quick(email, &uusage, &dusage);
+   if (ret) {
+	  if (uusage != -1) {
+		 /*
+			Convert quota string to integers
+		 */
+
+		 quota_mtos(quota_type, &usquota, &ucquota);
+
+		 /*
+			Return percentage
+		 */
+
+		 fd = (int)((long double)((long double)uusage/(long double)usquota) * (long double)100.0);
+		 if (fd > 100)
+			fd = 100;
+
+		 if (fd < 0)
+			fd = 0;
+
+		 return fd;
+	  }
+
+	  return 0;
+   }
+
+   /*
+	  If usage daemon is not running, fall back on old methods
+   */
 
 	(void)docheckquota(dir, &fd, quota_type, 0, 0, &percentage);
 	if (fd >= 0)
@@ -456,16 +556,16 @@ int	fd=-1;
 static int docheckquota(const char *dir,
 	int *maildirsize_fdptr,
 	const char *quota_type,
-	long xtra_size,
-	int xtra_cnt,
+	storage_t xtra_size,
+	storage_t xtra_cnt,
 	int *percentage)
 {
 char	*checkfolder=(char *)malloc(strlen(dir)+sizeof("/maildirfolder"));
 char	*newmaildirsizename;
 struct stat stat_buf;
-int	maildirsize_fd;
-off_t	maildirsize_size;
-int     maildirsize_cnt;
+int	maildirsize_fd = -1;
+storage_t maildirsize_size;
+storage_t maildirsize_cnt;
 unsigned maildirsize_nlines;
 int	n;
 time_t	tm;
@@ -507,6 +607,8 @@ struct dirent *de;
 			return (n);
 	}
 
+        /* rebuild the maildirsizefile  */
+
 	maxtime=0;
 	maildirsize_size=0;
 	maildirsize_cnt=0;
@@ -548,15 +650,16 @@ struct dirent *de;
 		return (-1);
 	}
 
+	chown(newmaildirsizename, VPOPMAILUID, VPOPMAILGID);
 	*maildirsize_fdptr=maildirsize_fd;
 
 	if (doaddquota(dir, maildirsize_fd, quota_type, maildirsize_size,
 		maildirsize_cnt, 1))
 	{
-		free(newmaildirsizename);
-		unlink(newmaildirsizename);
 		close(maildirsize_fd);
+		unlink(newmaildirsizename);
 		*maildirsize_fdptr= -1;
+		free(newmaildirsizename);
 		free(checkfolder);
 		return (-1);
 	}
@@ -565,7 +668,6 @@ struct dirent *de;
 
 	if (rename(newmaildirsizename, checkfolder))
 	{
-		free(checkfolder);
 		unlink(newmaildirsizename);
 		close(maildirsize_fd);
 		*maildirsize_fdptr= -1;
@@ -618,15 +720,30 @@ struct dirent *de;
 }
 
 int	maildir_addquota(const char *dir, int maildirsize_fd,
-	const char *quota_type, long maildirsize_size, int maildirsize_cnt)
+	const char *quota_type, storage_t maildirsize_size, storage_t maildirsize_cnt)
 {
+   int ret = 0;
+
+   /*
+	  Ping the usage daemon
+	  If it's running, we're done here.
+   */
+
+   ret = client_query_quick(" ", NULL, NULL);
+   if (ret)
+	  return 0;
+
+   /*
+	  Fall back
+   */
+
 	if (!quota_type || !*quota_type)	return (0);
 	return (doaddquota(dir, maildirsize_fd, quota_type, maildirsize_size,
 			maildirsize_cnt, 0));
 }
 
 static int doaddquota(const char *dir, int maildirsize_fd,
-	const char *quota_type, long maildirsize_size, int maildirsize_cnt,
+	const char *quota_type, storage_t maildirsize_size, storage_t maildirsize_cnt,
 	int isnew)
 {
 union	{
@@ -689,8 +806,7 @@ int	n;
 		niov=2;
 	}
 
-
-	sprintf(u.buf, "%ld %d\n", maildirsize_size, maildirsize_cnt);
+	sprintf(u.buf, "%llu %llu\n", maildirsize_size, maildirsize_cnt);
 	iov[niov].iov_base=u.buf;
 	iov[niov].iov_len=strlen(u.buf);
 
@@ -812,8 +928,7 @@ int	n;
 	return (n);
 }
 
-static int countcurnew(const char *dir, time_t *maxtime,
-	off_t *sizep, int *cntp)
+static int countcurnew(const char *dir, time_t *maxtime, storage_t *sizep, storage_t *cntp)
 {
 char	*p=(char *)malloc(strlen(dir)+5);
 int	n;
@@ -830,8 +945,7 @@ int	n;
 	return (n);
 }
 
-static int countsubdir(const char *dir, const char *subdir, time_t *maxtime,
-	off_t *sizep, int *cntp)
+static int countsubdir(const char *dir, const char *subdir, time_t *maxtime, storage_t *sizep, storage_t *cntp)
 {
 char	*p;
 int	n;
@@ -848,14 +962,13 @@ int	n;
 	return (n);
 }
 
-static int docount(const char *dir, time_t *dirstamp,
-	off_t *sizep, int *cntp)
+static int docount(const char *dir, time_t *dirstamp, storage_t *sizep, storage_t *cntp)
 {
 struct	stat	stat_buf;
 char	*p;
 DIR	*dirp;
 struct dirent *de;
-unsigned long	s;
+storage_t s;
 
 	if (stat(dir, &stat_buf))	return (0);	/* Ignore */
 	if (stat_buf.st_mtime > *dirstamp)	*dirstamp=stat_buf.st_mtime;
@@ -885,7 +998,7 @@ unsigned long	s;
 
 
 		if (maildir_parsequota(n, &s) == 0)
-			stat_buf.st_size=s;
+		   *sizep += s;
 		else
 		{
 			p=(char *)malloc(strlen(dir)+strlen(n)+2);
@@ -901,8 +1014,8 @@ unsigned long	s;
 				continue;
 			}
 			free(p);
+			*sizep += stat_buf.st_size;
 		}
-		*sizep += stat_buf.st_size;
 		++*cntp;
 	}
 
@@ -973,7 +1086,7 @@ char    *p=buf+sizeof(buf)-1;
         return (strcpy(arg, p));
 }
 
-static int maildir_parsequota(const char *n, unsigned long *s)
+static int maildir_parsequota(const char *n, storage_t *s)
 {
 const char *o;
 int     yes;
@@ -999,7 +1112,115 @@ int     yes;
                 *s=0;
                 while (*o >= '0' && *o <= '9')
                         *s= *s*10 + (*o++ - '0');
+
                 return (0);
         }
         return (-1);
 }
+
+/*
+   Converts a Maildir++ quota to storage_t values
+   Does not perform full syntax checking on quota format
+*/
+
+int quota_mtos(const char *quota, storage_t *size, storage_t *count)
+{
+   storage_t ts = 0;
+   const char *h = NULL, *t = NULL;
+
+   if (quota == NULL)
+	  return 0;
+
+   /*
+	  Set default values
+   */
+
+   if (size != NULL)
+	  *size = 0;
+
+   if (count != NULL)
+	  *count = 0;
+
+   /*
+	  Parse out seperate Maildir++ parts
+   */
+
+   h = t = quota;
+
+   while(1) {
+	  if ((*h == ',') || (!(*h))) {
+		 switch(*(h - 1)) {
+			case 'S':
+			   if (size) {
+				  ts = strtoll(t, NULL, 10);
+				  if (ts != -1)
+					 *size = ts;
+
+				  size = NULL;
+			   }
+
+			   break;
+
+			case 'C':
+			   if (count) {
+				  ts = strtoll(t, NULL, 10);
+				  if (ts != -1)
+					 *count = ts;
+
+				  count = NULL;
+			   }
+
+			   break;
+
+			default:
+			   /*
+				  Default is type S
+			   */
+
+			   if ((!(*h)) && (size)) {
+				  ts = strtoll(t, NULL, 10);
+				  if (ts != -1)
+					 *size = ts;
+
+				  size = NULL;
+			   }
+
+			   /*
+				  Unknown type
+			   */
+
+			   break;
+		 }
+
+		 if (!(*h))
+			break;
+
+		 while(*h == ',')
+			h++;
+
+		 t = h;
+	  }
+
+	  else
+		 h++;
+   }
+
+   return 1;
+}
+
+/*
+   Returns disk usage information for user and user's domain from
+   the vpopmail usage daemon
+*/
+
+int quota_user_usage(const char *user, storage_t *uusage, storage_t *dusage)
+{
+   int ret = 0;
+
+   if ((user == NULL) || (uusage == NULL) || (dusage == NULL) || (!(*user)))
+	  return 0;
+
+   ret = client_query_quick(user, uusage, dusage);
+   return ret;
+}
+
