@@ -1,10 +1,15 @@
-/* $Id: maildirquota.h,v 1.7 2007/09/23 22:59:53 rwidmer Exp $
+/* 
+   $Id: maildirquota.h 1014 2011-02-03 16:04:37Z volz0r $
+   Copyright (C) 2009 Inter7 Internet Technologies, Inc.
+ 
    This is a composite of deliverquota's maildirquota.h, maildirmisc.h, and 
    numlib.h.  I only consolidated them to keep this patch to vpopmail  a bit 
    less intrusive.
    -Bill Shupp
  */
 
+#include "storage.h"
+
 #define QUOTA_WARN_PERCENT 90
 
 /* I've removed pretty much the whole file execept for
@@ -12,8 +17,8 @@
    I"ve made the courier functions static.
    - Brian Kolaci
 */
-int readdomainquota(const char *dir, long *sizep, int *cntp);
-int readuserquota(const char* dir, long *sizep, int *cntp);
+int readdomainquota(const char *dir, storage_t *sizep, storage_t *cntp);
+int readuserquota(const char* dir, storage_t *sizep, storage_t *cntp);
 int domain_over_maildirquota(const char *userdir);
 int user_over_maildirquota(const char *dir, const char *quota);
 int vmaildir_readquota(const char *dir,	const char *quota);
@@ -21,8 +26,8 @@ int vmaildir_readquota(const char *dir,	const char *quota);
 int maildir_addquota(const char *,	/* Pointer to the maildir */
 	int,	/* Must be the int pointed to by 2nd arg to checkquota */
 	const char *,	/* The quota */
-	long,	/* +/- bytes */
-	int);	/* +/- files */
+	storage_t,	/* +/- bytes */
+	storage_t);	/* +/- files */
 
 /* skip the rest... */
 #if 0
@@ -45,7 +50,7 @@ int maildir_addquota(const char *,	/* Pointer to the maildir */
 extern "C" {
 #endif
 
-static const char maildirquota_h_rcsid[]="$Id: maildirquota.h,v 1.7 2007/09/23 22:59:53 rwidmer Exp $";
+static const char maildirquota_h_rcsid[]="$Id: maildirquota.h 1014 2011-02-03 16:04:37Z volz0r $";
 
 int maildir_checkquota(const char *,	/* Pointer to directory */
 	int *,	/* Initialized to -1, or opened descriptor for maildirsize */
@@ -84,7 +89,7 @@ int maildir_parsequota(const char *, unsigned long *);
 extern "C" {
 #endif
 
-static const char maildirmisc_h_rcsid[]="$Id: maildirquota.h,v 1.7 2007/09/23 22:59:53 rwidmer Exp $";
+static const char maildirmisc_h_rcsid[]="$Id: maildirquota.h 1014 2011-02-03 16:04:37Z volz0r $";
 
 /*
 **
@@ -235,7 +240,7 @@ int maildir_hasflag(const char *filename, char);
 extern "C" {
 #endif
 
-static const char numlib_h_rcsid[]="$Id: maildirquota.h,v 1.7 2007/09/23 22:59:53 rwidmer Exp $";
+static const char numlib_h_rcsid[]="$Id: maildirquota.h 1014 2011-02-03 16:04:37Z volz0r $";
 
 #define	NUMBUFSIZE	60
 
