@@ -1,6 +1,6 @@
 /*
- * $Id: vchkpw.c,v 1.21 2007/10/01 22:31:06 rwidmer Exp $
- * Copyright (C) 1999-2004 Inter7 Internet Technologies, Inc.
+ * $Id: vchkpw.c 1014 2011-02-03 16:04:37Z volz0r $
+ * Copyright (C) 1999-2009 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -160,9 +160,9 @@ int main( int argc, char **argv)
        * Note that the popular Courier-IMAP does not use vchkpw, it
        * links libvpopmail directly into its server.
        */
-      if (strstr (argv[1], "true") != NULL)  /* used as STMP AUTH */
+      if (strstr (argv[1] ? argv[1] : "", "true") != NULL)  /* used as STMP AUTH */
         ConnType = SMTP_CONN;
-      else if (strstr (argv[1], "imap") != NULL)  /* used with IMAP */
+      else if (strstr (argv[1] ? argv[1] : "", "imap") != NULL)  /* used with IMAP */
         ConnType = IMAP_CONN;
       else  /* default to POP */
         ConnType = POP_CONN;
@@ -689,20 +689,20 @@ void vlog(int verror, char *TheUser, char *TheDomain, char *ThePass,
   if ( (verror == VLOG_ERROR_PASSWD) && 
        ( ENABLE_LOGGING==1 || ENABLE_LOGGING==2 || ENABLE_LOGGING==3 || 
          ENABLE_LOGGING==4 ) ) {
-    syslog(LOG_NOTICE,sysc(LogLine));
+    syslog(LOG_NOTICE,"%s",sysc(LogLine));
 
   } else if ( verror == VLOG_ERROR_INTERNAL ) {
-    syslog(LOG_NOTICE, sysc(LogLine));
+    syslog(LOG_NOTICE, "%s",sysc(LogLine));
 
   } else if ( verror == VLOG_ERROR_LOGON ) {
-    syslog(LOG_NOTICE, sysc(LogLine));
+    syslog(LOG_NOTICE, "%s",sysc(LogLine));
 
   } else if ( verror == VLOG_ERROR_ACCESS ) {
-    syslog(LOG_NOTICE, sysc(LogLine));
+    syslog(LOG_NOTICE, "%s",sysc(LogLine));
 
   } else if ( verror == VLOG_AUTH && 
             ( ENABLE_LOGGING == 1 || ENABLE_LOGGING == 4 ) ) {
-    syslog(LOG_NOTICE, sysc(LogLine));
+    syslog(LOG_NOTICE, "%s",sysc(LogLine));
   }
 
 #ifdef ENABLE_SQL_LOGGING
@@ -712,26 +712,26 @@ void vlog(int verror, char *TheUser, char *TheDomain, char *ThePass,
 
   if ( (verror == VLOG_ERROR_PASSWD) && ( ENABLE_LOGGING==1 || ENABLE_LOGGING==2 || ENABLE_LOGGING==3 || ENABLE_LOGGING==4 ) ) {
       if ( (logsql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
-          syslog(LOG_NOTICE,"vchkpw: can't write SQL logs");
+          syslog(LOG_NOTICE,"%s","vchkpw: can't write SQL logs");
       }
       if ( (logsql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
-          syslog(LOG_NOTICE,"vchkpw: can't write SQL logs");
+          syslog(LOG_NOTICE,"%s","vchkpw: can't write SQL logs");
       }
   } else if ( verror == VLOG_ERROR_INTERNAL ) {
       if ( (logsql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
-        syslog(LOG_NOTICE,"vchkpw: can't write SQL logs");
+        syslog(LOG_NOTICE,"%s","vchkpw: can't write SQL logs");
       }
   } else if ( verror == VLOG_ERROR_LOGON ) {
       if ( (logsql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
-        syslog(LOG_NOTICE,"vchkpw: can't write SQL logs");
+        syslog(LOG_NOTICE,"%s","vchkpw: can't write SQL logs");
       }
   } else if ( verror == VLOG_ERROR_ACCESS ) {
       if ( (logsql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
-        syslog(LOG_NOTICE,"vchkpw: can't write SQL logs");
+        syslog(LOG_NOTICE,"%s","vchkpw: can't write SQL logs");
       }
   } else if ( verror == VLOG_AUTH && ( ENABLE_LOGGING == 1 || ENABLE_LOGGING == 4 ) ) {
       if ( (logsql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
-        syslog(LOG_NOTICE,"vchkpw: can't write SQL logs");
+        syslog(LOG_NOTICE,"%s","vchkpw: can't write SQL logs");
       }
   }
 #endif
