@@ -1,6 +1,6 @@
 /*
- * $Id: vdeldomain.c,v 1.11 2007/05/22 03:59:00 rwidmer Exp $
- * Copyright (C) 1999-2004 Inter7 Internet Technologies, Inc.
+ * $Id: vdeldomain.c 1015 2011-02-03 16:33:39Z volz0r $
+ * Copyright (C) 1999-2009 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,9 +42,9 @@ int main(int argc, char *argv[])
  int err=0;
 
  domain_entry *entry;
- char *aliases[MAX_DOM_ALIAS];
+ string_list aliases;
  char parent[MAX_BUFF];
- int  i, aliascount=0, doit=1;
+ int  i, doit=1;
 
     if( vauth_open( 1 )) {
         vexiterror( stderr, "Initial open." );
@@ -63,9 +63,11 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	string_list_init(&aliases, 10);
+
 	while( entry ) {
 		if (strcmp(entry->domain, entry->realdomain) != 0) {
-			aliases[aliascount++] = strdup(entry->domain);
+			string_list_add(&aliases, entry->domain);
 		} else {
 			snprintf(parent,sizeof(parent),"%s",entry->domain);
 		}
@@ -73,7 +75,7 @@ int main(int argc, char *argv[])
 		entry = get_domain_entries(NULL);
 	}
 
-	if( aliascount > 0 && 0 == strncmp(Domain,parent,MAX_BUFF)) {  
+	if( aliases.count > 0 && 0 == strncmp(Domain,parent,MAX_BUFF)) {  
 		//  Have aliases
 		if( force ) {
 			printf("Warning: Alias domains deleted:\n");
@@ -82,12 +84,13 @@ int main(int argc, char *argv[])
 			doit=0;
 		}
 
-		for(i=0;i<aliascount;i++) {
-			printf ("   %s\n", aliases[i]);
-			free( aliases[i] );
+		for(i=0;i<aliases.count;i++) {
+			printf ("   %s\n", aliases.values[i]);
 		} 
 	}
 
+	string_list_free(&aliases);
+
         if( doit ) {
 	if ( (err=vdeldomain(Domain)) != VA_SUCCESS) {
 		printf("Error: %s\n", verror(err));
