@@ -1,6 +1,6 @@
 /*
- * $Id: vdelivermail.c,v 1.18 2007/05/22 03:59:00 rwidmer Exp $
- * Copyright (C) 1999-2003 Inter7 Internet Technologies, Inc.
+ * $Id: vdelivermail.c 1014 2011-02-03 16:04:37Z volz0r $
+ * Copyright (C) 1999-2009 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -821,6 +821,8 @@ int check_forward_deliver(char *dir)
     chdir(dir);
 
 #ifdef QMAIL_EXT
+    fs = NULL;
+
     /* format the file name */
     if (strlen(TheExt)) {
         strcpy(tmpbuf,".qmail-");
@@ -837,9 +839,10 @@ int check_forward_deliver(char *dir)
                 }
             }
         }
-    } else {
-        fs = fopen(".qmail","r");
     }
+	
+	if (fs == NULL)
+        fs = fopen(".qmail","r");
 #else
     fs = fopen(".qmail","r");
 #endif
@@ -895,6 +898,9 @@ void (*f)();
  */
 void run_command(char *prog)
 {
+
+#define MAX_ENV_BUFF 100
+
  int child;
  char *(args[4]);
  int wstat;
@@ -907,7 +913,8 @@ void run_command(char *prog)
      printf("Unable to fork: %d.", errno); 
      vexit(EXIT_DEFER);
    case 0:
-     setenv("SHELL", "/bin/sh", 1);
+     
+     putenv("SHELL=/bin/sh");
      args[0] = "/bin/sh"; args[1] = "-c"; args[2] = prog; args[3] = 0;
      sig_catch(SIGPIPE,SIG_DFL);
      execv(*args,args);
