@@ -1,6 +1,6 @@
 /*
- * $Id: vdominfo.c,v 1.15 2007/05/22 03:59:00 rwidmer Exp $
- * Copyright (C) 2001-2004 Inter7 Internet Technologies, Inc.
+ * $Id: vdominfo.c 1015 2011-02-03 16:33:39Z volz0r $
+ * Copyright (C) 2001-2009 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@ int DisplayGid;
 int DisplayDir;
 int DisplayAll;
 int DisplayTotalUsers;
+int DisplayRealDomain;
 
 void usage();
 void get_options(int argc, char **argv);
@@ -92,11 +93,12 @@ void get_options(int argc, char **argv)
     DisplayDir = 0;
     DisplayTotalUsers = 0;
     DisplayAll = 1;
+	DisplayRealDomain = 0;
 
     memset(Domain, 0, sizeof(Domain));
 
     errflag = 0;
-    while( !errflag && (c=getopt(argc,argv,"vanugdt")) != -1 ) {
+    while( !errflag && (c=getopt(argc,argv,"vanugdtr")) != -1 ) {
         switch(c) {
             case 'v':
                 printf("version: %s\n", VERSION);
@@ -124,6 +126,10 @@ void get_options(int argc, char **argv)
             case 'a':
                 DisplayAll = 1;
                 break;
+            case 'r':
+                DisplayRealDomain = 1;
+				DisplayAll = 0;
+                break;
             default:
                 errflag = 1;
                 break;
@@ -169,6 +175,8 @@ void display_domain(char *domain, char *dir, uid_t uid, gid_t gid, char *realdom
             printf("%lu\n",  vdir.cur_users);
             close_big_dir(realdomain,uid,gid);
         }
+
+		if ( DisplayRealDomain ) printf("%s\n", realdomain);
     }
 }
 
@@ -199,8 +207,8 @@ void display_all_domains()
 void display_one_domain( char * Domain )
 {
  domain_entry *entry;
- char *aliases[MAX_DOM_ALIAS];
- int  i, aliascount=0;
+ string_list aliases;
+ int  i;
 
     entry = get_domain_entries( Domain );
     if (entry==NULL) {
@@ -213,11 +221,13 @@ void display_one_domain( char * Domain )
       }
     }
 
+    string_list_init(&aliases, 10);
+
     while( entry ) {
 	if (strcmp(entry->domain, entry->realdomain) != 0) {
 // 		printf ("Note:   %s is an alias for %s\n",
 //                         entry->domain, entry->realdomain);
-                aliases[aliascount++] = strdup(entry->domain);
+                string_list_add(&aliases, entry->domain);
 
         } else {
 		display_domain(entry->domain, entry->path, entry->uid, 
@@ -227,9 +237,9 @@ void display_one_domain( char * Domain )
         entry = get_domain_entries(NULL);
     }
 
-    for(i=0;i<aliascount;i++) {
- 	printf ("alias: %s\n", aliases[i]);
-        free( aliases[i] );
+    for(i=0;i<aliases.count;i++) {
+ 	printf ("alias: %s\n", aliases.values[i]);
     } 
+    string_list_free(&aliases);
 }
 
