@@ -1,5 +1,7 @@
 /*
- * $Id: vlimits.h,v 1.10 2007/05/22 03:59:00 rwidmer Exp $
+ * $Id: vlimits.h 1026 2011-02-08 21:35:17Z volz0r $
+ * Copyright (C) 2009 Inter7 Internet Technologies, Inc.
+ *
  * handle domain limits in both file and mysql tables
  * Brian Kolaci <bk@galaxy.net>
  */
@@ -15,6 +17,8 @@
 #define VLIMIT_DISABLE_ALL (VLIMIT_DISABLE_CREATE|VLIMIT_DISABLE_MODIFY|VLIMIT_DISABLE_DELETE)
 #define VLIMIT_DISABLE_BITS 3
 
+#include "storage.h"
+
 struct vlimits {
       /* max service limits */
       int       maxpopaccounts;
@@ -24,10 +28,10 @@ struct vlimits {
       int       maxmailinglists;
 
       /* quota & message count limits */
-      int       diskquota;
-      int       maxmsgcount;
-      int       defaultquota;
-      int       defaultmaxmsgcount;
+      storage_t diskquota;
+      storage_t maxmsgcount;
+      storage_t defaultquota;
+      storage_t defaultmaxmsgcount;
 
       /* the following are 0 (false) or 1 (true) */
       short     disable_pop;
@@ -49,8 +53,8 @@ struct vlimits {
       short     perm_maillist;
       short     perm_maillist_users;
       short     perm_maillist_moderators;
-      short     perm_quota;
-      short     perm_defaultquota;
+      short		perm_quota;
+      short		perm_defaultquota;
 };
 
 void vdefault_limits(struct vlimits *limits);
