@@ -1,6 +1,6 @@
 /*
- * $Id: vmysql.c,v 1.35 2007/08/17 22:49:23 rwidmer Exp $
- * Copyright (C) 1999-2004 Inter7 Internet Technologies, Inc.
+ * $Id: vmysql.c 1026 2011-02-08 21:35:17Z volz0r $
+ * Copyright (C) 1999-2009 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@ Add error result for "unable to read vpopmail.mysql" and return it
 #include <string.h>
 #include <time.h>
 #include <mysql.h>
+#include <mysqld_error.h>
 #include "config.h"
 #include "vpopmail.h"
 #include "vauth.h"
@@ -167,12 +168,12 @@ int load_connection_info() {
         MYSQL_UPDATE_SERVER = strtok(conf_update, delimiters);
         if (MYSQL_UPDATE_SERVER == NULL) return VA_PARSE_ERROR06;
   
-  	MYSQL_READ_SOCKET = strtok(NULL, delimiters);
-  	if (MYSQL_READ_SOCKET == NULL) return VA_PARSE_ERROR;
+  	MYSQL_UPDATE_SOCKET = strtok(NULL, delimiters);
+  	if (MYSQL_UPDATE_SOCKET == NULL) return VA_PARSE_ERROR;
   
-  	if (MYSQL_READ_SOCKET[0] != '/') {
-  	    MYSQL_READ_PORT = atoi(MYSQL_READ_SOCKET);
-  	    MYSQL_READ_SOCKET = NULL;
+  	if (MYSQL_UPDATE_SOCKET[0] != '/') {
+  	    MYSQL_UPDATE_PORT = atoi(MYSQL_UPDATE_SOCKET);
+  	    MYSQL_UPDATE_SOCKET = NULL;
   	}
   
         MYSQL_UPDATE_USER = strtok(NULL, delimiters);
@@ -316,11 +317,11 @@ int vauth_open_read_getall()
     mysql_init(&mysql_read_getall);
     if (!(mysql_real_connect(&mysql_read_getall, MYSQL_READ_SERVER, 
             MYSQL_READ_USER, MYSQL_READ_PASSWD, MYSQL_READ_DATABASE, 
-            MYSQL_READ_PORT, NULL, 0))) {
+            MYSQL_READ_PORT, MYSQL_READ_SOCKET, 0))) {
         /* we could not connect, at least try the update server */
         if (!(mysql_real_connect(&mysql_read_getall, MYSQL_UPDATE_SERVER, 
             MYSQL_UPDATE_USER, MYSQL_UPDATE_PASSWD, MYSQL_UPDATE_DATABASE, 
-            MYSQL_UPDATE_PORT, NULL, 0))) {
+            MYSQL_UPDATE_PORT, MYSQL_UPDATE_SOCKET, 0))) {
             verrori = VA_NO_AUTH_CONNECTION;
             return(-1);
         }
@@ -591,7 +592,9 @@ int vauth_deldomain( char *domain )
     qnprintf( SqlBufUpdate, SQL_BUF_SIZE, 
         "delete from lastauth where domain = '%s'", domain );
     if (mysql_query(&mysql_update,SqlBufUpdate)) {
-        return(-1);
+	   err = mysql_errno(&mysql_update);
+	   if (err != ER_NO_SUCH_TABLE)
+		  fprintf(stderr, "vauth_deldomain: warning: mysql_query(%s) failed: %s\n", SqlBufUpdate, mysql_error(&mysql_update));
     } 
 #endif
 
@@ -599,7 +602,9 @@ int vauth_deldomain( char *domain )
     qnprintf( SqlBufUpdate, SQL_BUF_SIZE,
        "delete from vlog where domain = '%s'", domain );
     if (mysql_query(&mysql_update,SqlBufUpdate)) {
-       return(-1);
+	   err = mysql_errno(&mysql_update);
+	   if (err != ER_NO_SUCH_TABLE)
+		  fprintf(stderr, "vauth_deldomain: warning: mysql_query(%s) failed: %s\n", SqlBufUpdate, mysql_error(&mysql_update));
     }
 #endif
 
@@ -642,7 +647,11 @@ int vauth_deluser( char *user, char *domain )
         "delete from lastauth where user = '%s' and domain = '%s'", 
         user, domain );
     if (mysql_query(&mysql_update,SqlBufUpdate)) {
-        err = -1;
+	   err = mysql_errno(&mysql_update);
+	   if (err != ER_NO_SUCH_TABLE)
+		  err = -1;
+	   else
+		  err = 0;
     } 
 #endif
 
@@ -651,7 +660,11 @@ int vauth_deluser( char *user, char *domain )
         "delete from vlog where domain = '%s' and user = '%s'", 
        domain, user );
     if (mysql_query(&mysql_update,SqlBufUpdate)) {
-        err = -1;
+	   err = mysql_errno(&mysql_update);
+	   if (err != ER_NO_SUCH_TABLE)
+		  err = -1;
+	   else
+		  err = 0;
     }
 #endif
     return(err);
@@ -792,11 +805,11 @@ void vauth_end_getall()
 char *vauth_munch_domain( char *domain )
 {
  int i;
- static char tmpbuf[50];
+ static char tmpbuf[512];
 
     if ( domain == NULL || domain[0] == 0 ) return(domain);
 
-    for(i=0;domain[i]!=0;++i){
+    for(i=0; ((domain[i]!=0) && (i < (sizeof(tmpbuf) - 1)));++i){
         tmpbuf[i] = domain[i];
         if ( domain[i] == '.' || domain[i] == '-' ) {
             tmpbuf[i] = MYSQL_DOT_CHAR;
@@ -1707,7 +1720,7 @@ int vget_limits(const char *domain, struct vlimits *limits)
         "maxmsgcount, defaultquota, defaultmaxmsgcount, "
         "disable_pop, disable_imap, disable_dialup, "
         "disable_passwordchanging, disable_webmail, disable_relay, "
-        "disable_smtp, disable_spamassassin, delete_spam, perm_account, "
+        "disable_smtp, disable_spamassassin, delete_spam, disable_maildrop, perm_account, "
         "perm_alias, perm_forward, perm_autoresponder, perm_maillist, "
         "perm_quota, perm_defaultquota \n"
         "FROM limits \n"
@@ -1742,10 +1755,10 @@ int vget_limits(const char *domain, struct vlimits *limits)
         limits->maxforwards = atoi(row[2]);
         limits->maxautoresponders = atoi(row[3]);
         limits->maxmailinglists = atoi(row[4]);
-        limits->diskquota = atoi(row[5]);
-        limits->maxmsgcount = atoi(row[6]);
-        limits->defaultquota = atoi(row[7]);
-        limits->defaultmaxmsgcount = atoi(row[8]);
+        limits->diskquota = strtoll(row[5], NULL, 10);
+        limits->maxmsgcount = strtoll(row[6], NULL, 10);
+        limits->defaultquota = strtoll(row[7], NULL, 10);
+        limits->defaultmaxmsgcount = strtoll(row[8], NULL, 10);
         limits->disable_pop = atoi(row[9]);
         limits->disable_imap = atoi(row[10]);
         limits->disable_dialup = atoi(row[11]);
@@ -1755,17 +1768,18 @@ int vget_limits(const char *domain, struct vlimits *limits)
         limits->disable_smtp = atoi(row[15]);
         limits->disable_spamassassin = atoi(row[16]);
         limits->delete_spam = atoi(row[17]);
-        limits->perm_account = atoi(row[18]);
-        limits->perm_alias = atoi(row[19]);
-        limits->perm_forward = atoi(row[20]);
-        limits->perm_autoresponder = atoi(row[21]);
+        limits->disable_maildrop = atoi(row[18]);
+        limits->perm_account = atoi(row[19]);
+        limits->perm_alias = atoi(row[20]);
+        limits->perm_forward = atoi(row[21]);
+        limits->perm_autoresponder = atoi(row[22]);
         limits->perm_maillist = perm & VLIMIT_DISABLE_ALL;
         perm >>= VLIMIT_DISABLE_BITS;
         limits->perm_maillist_users = perm & VLIMIT_DISABLE_ALL;
         perm >>= VLIMIT_DISABLE_BITS;
         limits->perm_maillist_moderators = perm & VLIMIT_DISABLE_ALL;
-        limits->perm_quota = atoi(row[23]);
-        limits->perm_defaultquota = atoi(row[24]);
+        limits->perm_quota = strtoll(row[23], NULL, 10);
+        limits->perm_defaultquota = strtoll(row[24], NULL, 10);
     }
     mysql_free_result(res_read);
 
@@ -1785,11 +1799,11 @@ int vset_limits(const char *domain, const struct vlimits *limits)
         "diskquota, maxmsgcount, defaultquota, defaultmaxmsgcount, "
         "disable_pop, disable_imap, disable_dialup, "
         "disable_passwordchanging, disable_webmail, disable_relay, "
-        "disable_smtp, disable_spamassassin, delete_spam, perm_account, "
+        "disable_smtp, disable_spamassassin, delete_spam, disable_maildrop, perm_account, "
         "perm_alias, perm_forward, perm_autoresponder, perm_maillist, "
         "perm_quota, perm_defaultquota) \n"
         "VALUES \n"
-        "('%s', %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d)",
+        "('%s', %d, %d, %d, %d, %d, %S, %S, %S, %S, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d)",
         domain,
         limits->maxpopaccounts,
         limits->maxaliases,
@@ -1809,6 +1823,7 @@ int vset_limits(const char *domain, const struct vlimits *limits)
         limits->disable_smtp,
         limits->disable_spamassassin,
         limits->delete_spam,
+        limits->disable_maildrop,
         limits->perm_account,
         limits->perm_alias,
         limits->perm_forward,
@@ -1818,7 +1833,7 @@ int vset_limits(const char *domain, const struct vlimits *limits)
          (limits->perm_maillist_moderators << (VLIMIT_DISABLE_BITS * 2))),
         limits->perm_quota,
         limits->perm_defaultquota);
-
+	
     if (mysql_query(&mysql_update,SqlBufUpdate)) {
         vcreate_limits_table();
         if (mysql_query(&mysql_update,SqlBufUpdate)) {
