@@ -1,6 +1,6 @@
 /*
- * $Id: vmysql.h,v 1.7 2007/05/22 03:59:01 rwidmer Exp $
- * Copyright (C) 1999 Inter7 Internet Technologies, Inc.
+ * $Id: vmysql.h 1026 2011-02-08 21:35:17Z volz0r $
+ * Copyright (C) 1999-2009 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -283,10 +283,10 @@ valias_line text not null, index (alias, domain)"
       maxforwards              INT(10) NOT NULL DEFAULT -1, \
       maxautoresponders        INT(10) NOT NULL DEFAULT -1, \
       maxmailinglists          INT(10) NOT NULL DEFAULT -1, \
-      diskquota                INT(12) NOT NULL DEFAULT 0, \
-      maxmsgcount              INT(12) NOT NULL DEFAULT 0, \
-      defaultquota             INT(12) NOT NULL DEFAULT 0, \
-      defaultmaxmsgcount       INT(12) NOT NULL DEFAULT 0, \
+      diskquota                BIGINT UNSIGNED NOT NULL DEFAULT 0, \
+      maxmsgcount              BIGINT UNSIGNED NOT NULL DEFAULT 0, \
+      defaultquota             BIGINT UNSIGNED NOT NULL DEFAULT 0, \
+      defaultmaxmsgcount       BIGINT UNSIGNED NOT NULL DEFAULT 0, \
       disable_pop              TINYINT(1) NOT NULL DEFAULT 0, \
       disable_imap             TINYINT(1) NOT NULL DEFAULT 0, \
       disable_dialup           TINYINT(1) NOT NULL DEFAULT 0, \
@@ -296,6 +296,7 @@ valias_line text not null, index (alias, domain)"
       disable_smtp             TINYINT(1) NOT NULL DEFAULT 0, \
       disable_spamassassin     TINYINT(1) NOT NULL DEFAULT 0, \
       delete_spam              TINYINT(1) NOT NULL DEFAULT 0, \
+      disable_maildrop		   TINYINT(1) NOT NULL DEFAULT 0, \
       perm_account             TINYINT(2) NOT NULL DEFAULT 0, \
       perm_alias               TINYINT(2) NOT NULL DEFAULT 0, \
       perm_forward             TINYINT(2) NOT NULL DEFAULT 0, \
