@@ -1,6 +1,6 @@
 /*
- * $Id: vpgsql.c,v 1.39 2007/08/17 22:49:23 rwidmer Exp $
- * Copyright (C) 1999-2004 Inter7 Internet Technologies, Inc.
+ * $Id: vpgsql.c 1014 2011-02-03 16:04:37Z volz0r $
+ * Copyright (C) 1999-2009 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -226,19 +226,21 @@ int vauth_adduser(char *user, char *domain, char *pass, char *gecos,
     domstr = PGSQL_LARGE_USERS_TABLE;
   }
 
+  *dirbuf = '\0';
+
   if ( strlen(domain) <= 0 ) {
     if ( strlen(dir) > 0 ) {
-      snprintf(dirbuf, SQL_BUF_SIZE, 
+      snprintf(dirbuf, sizeof(dirbuf), 
 	       "%s/users/%s/%s", VPOPMAILDIR, dir, user);
     } else {
-      snprintf(dirbuf, SQL_BUF_SIZE, "%s/users/%s", VPOPMAILDIR, user);
+      snprintf(dirbuf, sizeof(dirbuf), "%s/users/%s", VPOPMAILDIR, user);
     }
   } else {
     vget_assign(domain, dom_dir, 156, &uid, &gid );
     if ( strlen(dir) > 0 ) {
-      snprintf(dirbuf,SQL_BUF_SIZE, "%s/%s/%s", dom_dir, dir, user);
+      snprintf(dirbuf,sizeof(dirbuf), "%s/%s/%s", dom_dir, dir, user);
     } else {
-      snprintf(dirbuf,SQL_BUF_SIZE, "%s/%s", dom_dir, user);
+      snprintf(dirbuf,sizeof(dirbuf), "%s/%s", dom_dir, user);
     }
   }
 
@@ -248,7 +250,7 @@ int vauth_adduser(char *user, char *domain, char *pass, char *gecos,
     Crypted[0] = 0;
   }
 
-  qnprintf( SqlBufUpdate, SQL_BUF_SIZE, INSERT, 
+  qnprintf( SqlBufUpdate, sizeof(SqlBufUpdate), INSERT, 
 	    domstr, user, 
 #ifdef MANY_DOMAINS
 	    domain,
@@ -587,11 +589,11 @@ void vauth_end_getall()
 char *vauth_munch_domain( char *domain )
 {
   int i;
-  static char tmpbuf[50];
+  static char tmpbuf[512];
 
   if ( domain == NULL || domain[0] == 0 ) return(domain);
 
-  for(i=0;domain[i]!=0;++i){
+  for(i=0;domain[i]!=0 && i < (sizeof(tmpbuf) - 1);++i){
     tmpbuf[i] = tolower(domain[i]);
     if ( domain[i] == '.' || domain[i] == '-' ) {
       tmpbuf[i] = SQL_DOT_CHAR;
