@@ -1,6 +1,6 @@
 /*
- * $Id: vpopmail.h,v 1.32 2007/09/29 23:17:35 rwidmer Exp $
- * Copyright (C) 1999-2004 Inter7 Internet Technologies, Inc.
+ * $Id: vpopmail.h 1015 2011-02-03 16:33:39Z volz0r $
+ * Copyright (C) 1999-2009 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 #include <stdio.h>
 #include <sys/types.h>		// for uid_t
+#include "storage.h"
 
 /*  Enable expanded debug information.  Consider these for ./configure options  */
 //  Show entry and parms when hitting vpopmail library functions
@@ -34,7 +35,6 @@
 
 /* max buffer sizes */
 #define MAX_BUFF 300
-#define MAX_DOM_ALIAS 100
 
 /* max field sizes */
 #define MAX_PW_NAME          32
@@ -139,37 +139,166 @@
 #define SA_EXPERT    0x20000
 #define NO_MAILDROP  0x40000
 
+/*  Constants for vlistlib  */
+#define MAX_SORT_BLOCKS 1000
+#define SORT_TABLE_ENTRIES 1000
+#define MAX_OPTIONS 26
+#define MAX_FILE_NAME 512
+#define MAX_NAME 160
+#define EZMLMDIR "/usr/local/bin/ezmlm"
+
+#define POSTING_ANYONE 0
+#define POSTING_SPOB 1
+#define POSTING_SPOM 2
+#define POSTING_MPOB 3
+#define POSTING_MPOM 4
+
+#define REPLYTO_SENDER 1
+#define REPLYTO_LIST 2
+#define REPLYTO_ADDRESS 3
+
+/*   List of strings */
+typedef struct string_list {
+    int size;
+    int count;
+    char **values;
+} string_list;
+
+void string_list_init(string_list *a, int first);
+int string_list_add(string_list *a, char *value);
+void string_list_free(string_list *a);
+
+/*   Domain entry  */
+typedef struct domain_entry {
+        char    *domain;
+        char    *realdomain;
+        int             uid;
+        int             gid;
+        char    *path;
+} domain_entry;
+
+
+/*   Structure for mailing list data  */
+typedef struct listInfoType {
+   domain_entry *entry;
+   char Name[MAX_NAME];
+   int Posting;
+   int Access;
+   int Archive;
+   int DomainOnly;
+   int Prefix;
+   int ReplyTo;
+   int Digest;
+   int Edit;
+   int SubConf;
+   int Indexed;
+   int UnsubConf;
+   int Kill;
+   int RemoteSub;
+   int RemoteText;
+   int Public;
+   int Requests;
+   int Remote;
+   int SubMod;
+   int Trailer;
+   int Warn;
+   int Extra;
+   int SQLSupport;
+   int SQLPort;
+   char OwnerEmail[MAX_NAME];
+   char SQLHost[MAX_NAME];
+   char SQLUser[MAX_NAME];
+   char SQLPass[MAX_NAME];
+   char SQLBase[MAX_NAME];
+   char SQLTable[MAX_NAME];
+   char Dot[MAX_NAME];
+   char Dir[MAX_NAME];
+   char PrefixText[MAX_NAME];
+   char Domain[MAX_NAME];
+   char ReplyTo_Addr[MAX_NAME];
+} listInfoType;
 
 extern int OptimizeAddDomain;
 extern int NoMakeIndex;
 extern int verrori;
 
 /* functions */
+
+/* Alias Management */
+//  these may be in their own .h file...
+
+/* Domain Management */
 int vadddomain( char *domain, char *dir, uid_t uid, gid_t gid);
+int vaddaliasdomain( char *alias_domain, char *real_domain);
 int vdeldomain( char *);
+domain_entry *get_domain_entries( const char *match_real );
+
+
+/*  User Management  */
 int vadduser( char *, char *, char *, char *, int);
-int vdeluser( char *, char *);
+void vgetpasswd( char *, char *, size_t);
 int vpasswd( char *, char *, char *, int);
+int vdeluser( char *, char *);
 int vsetuserquota( char *, char *, char * );
+char *vget_assign(char *domain, char *dir, int dir_len, uid_t *uid, gid_t *gid);
+void remove_maildirsize(char *dir);
+void update_maildirsize(char *domain, char *dir, char *quota);
+
+
+/* Mailing List Management */
+//  Subs = Subscribers.  I really don't want to write that out every time...
+int    listSubsDescription( listInfoType *LI, int mode, int size, char *description );
+int    listSubsCount(       listInfoType *LI, int mode );
+char **listSubsList(        listInfoType *LI, int mode, int page, int perPage );
+int    listSubsAdd(         listInfoType *LI, int mode, char *email ); 
+int    listSubsDel(         listInfoType *LI, int mode, char *email ); 
+
+int    listCount(      listInfoType *LI );
+char **listList(       listInfoType *LI, int page, int perPage );
+int    listGetOptions( listInfoType *LI, char *options, int maxOptions );
+int    listMake(       listInfoType *LI );
+int    listDelete(     listInfoType *LI );
+int    listInit(       listInfoType *LI, char *address, int isDomainOnly, int isCreating );
+int    listClose(      listInfoType *LI );
+void   listGetError( char *buff, const int size, const int status );
+
+/*  Low level string support  */
+int parse_email( char *, char *, char *, int);
+char *vrandom_pass (char *buffer, int len);
+int is_username_valid( char *user );
+int is_domain_valid( char *domain );
+char *maildir_to_email(const char *maildir);
+
+
+/*  Operational Overhead  */
 int vexit(int err);
 int vexiterror( FILE *f, char *comment);
 
+
+/*  Utility Functions  */
+/*  Low Level File System Management  */
+int vdelfiles( char *);
+int r_chown( char *, uid_t, gid_t);
+
+
+/*  Misc Functions  */
+int vopen_smtp_relay();	
+char *get_remote_ip();
+
+
+/*  Not yet categorized  */
 char randltr(void);
 int mkpasswd3( char *, char *, int);
-void vgetpasswd( char *, char *, size_t);
-int vdelfiles( char *);
 int add_domain_assign( char *alias_domain, char *real_domain, 
                        char *dir, uid_t uid, gid_t gid);
 //int del_control( char *);
-int del_control( char *aliases[MAX_DOM_ALIAS], int aliascount);
-int del_domain_assign( char *aliasies[MAX_DOM_ALIAS], int aliascount, 
+int del_control( char **aliases, int aliascount);
+int del_domain_assign( char **aliasies, int aliascount, 
                        char *real_domain,
                        char *dir, uid_t uid, gid_t gid);
-int remove_lines( char *filename, char *aliases[MAX_DOM_ALIAS], int aliascount);
-int r_chown( char *, uid_t, gid_t);
+int remove_lines( char *filename, char **aliases, int aliascount);
 int signal_process( char *, int );
 int update_newu();
-int parse_email( char *, char *, char *, int);
 int add_user_assign( char *, char *);
 int del_user_assign( char *);
 void lowerit( char *);
@@ -182,17 +311,14 @@ int compile_morercpthosts();
 char *make_user_dir(char *username, char *domain, uid_t uid, gid_t gid);
 int r_mkdir(char *, uid_t uid, gid_t gid);
 struct vqpasswd *vgetent(FILE *);
-int pw_comp(char *, char *, char *, int);
 char *default_domain();
 void vset_default_domain( char *);
-int vopen_smtp_relay();	
 void vupdate_rules(int);
 void vclear_open_smtp(time_t, time_t);
 char *verror(int);
 int vadddotqmail(char *alias, char *domain,... ); 
 int vdeldotqmail( char *alias, char *domain);
 int vget_real_domain(char *domain, int len );
-char *vget_assign(char *domain, char *dir, int dir_len, uid_t *uid, gid_t *gid);
 struct vqpasswd *vauth_user(char *user, char *domain, char *password, char *apop);
 int vmake_maildir(char *domain, char *dir);
 int vsqwebmail_pass( char *dir, char *crypted, uid_t uid, gid_t gid );
@@ -202,21 +328,13 @@ int vfd_copy(int,int);
 int vfd_move(int,int);
 int update_rules();
 char *vversion(char *);
-void remove_maildirsize(char *dir);
-void update_maildirsize(char *domain, char *dir, char *quota);
 int vcheck_vqpw(struct vqpasswd *inpw, char *domain);
 char *vgen_pass(int len);
-char *vrandom_pass (char *buffer, int len);
 int vvalidchar( char inchar );
-int is_username_valid( char *user );
-int is_domain_valid( char *domain );
-int vaddaliasdomain( char *alias_domain, char *real_domain);
 char *format_maildirquota(const char *q);
 char *date_header();
-char *get_remote_ip();
-char *maildir_to_email(const char *maildir);
 int qnprintf (char *buffer, size_t size, const char *format, ...);
-int readuserquota(const char* dir, long *sizep, int *cntp);
+int readuserquota(const char* dir, storage_t *sizep, storage_t *cntp);
 #ifndef HAVE_WARN
 #define warn(...) fprintf(stderr, __VA_ARGS__)
 #endif
@@ -238,17 +356,6 @@ char *dec2hex(unsigned char *);
 
 #endif
 
-typedef struct domain_entry {
-        char    *domain;
-        char    *realdomain;
-        int             uid;
-        int             gid;
-        char    *path;
-        char    *aliases[MAX_DOM_ALIAS];
-} domain_entry;
-
-domain_entry *get_domain_entries( const char *match_real );
-
 void vsqlerror( FILE *f, char *comment );
 
 //  Error handling
@@ -261,3 +368,7 @@ extern char onchange_buf[MAX_BUFF];
 extern int allow_onchange;
 int call_onchange();
 #endif
+
+#ifdef USERS_BIG_DIR
+char *backfill(char *, char *, char *, int);
+#endif
