@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2004 Inter7 Internet Technologies, Inc.
+ * $Id: vpopmaild.c 1015 2011-02-03 16:33:39Z volz0r $
+ * Copyright (C) 2004-2009 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,7 +40,6 @@
 
 #define READ_TIMEOUT 60
 #define MAX_TMP_BUFF 1024
-#define MAX_FILE_NAME 156
 
 #define TOKENS " \n\t\r"
 #define PASS_TOKENS "\n\r"
@@ -172,7 +172,7 @@ func_t Functions[] = {
 {2, "Domain", NULL, NULL},
 {2, "user_info", user_info, "user_domain<crlf>" },
 {3, "list_domains", list_domains, "[page per_page]<crlf>" },
-{3, "find_domain", find_domain, "domain per-page<crlf>" },
+{3, "find_domain", find_domain, "domain [per-page]<crlf>" },
 {3, "domain_count", domain_count, "<crlf>" },
 {3, "add_alias_domain", add_alias_domain, "domain alias<crlf>" },
 {3, "add_domain", add_domain, "domain postmaster-password<crlf>" },
@@ -244,8 +244,8 @@ int wait_read()
 
   memset(ReadBuf,0,sizeof(ReadBuf));
   if (select(2,&rfds,(fd_set *) 0,(fd_set *)0,&tv)>=1) {
-    fgets(ReadBuf,sizeof(ReadBuf),stdin);
-    return(1);
+    if (fgets(ReadBuf,sizeof(ReadBuf),stdin) != NULL)
+	  return(1);
   }
   return(-1);
 }
@@ -294,7 +294,7 @@ int main(int argc, char **argv)
     exit( -1 );
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK);
   wait_write();
 
   /* authenticate first or drop connection */
@@ -435,8 +435,8 @@ int login()
   AuthVpw.pw_shell = strdup(tmpvpw->pw_shell);
   AuthVpw.pw_clear_passwd = strdup(tmpvpw->pw_clear_passwd);
 
-  snprintf( TheUserDir, sizeof(TheUserDir), AuthVpw.pw_dir);
-  snprintf( TheDomainDir, sizeof(TheDomainDir), 
+  snprintf( TheUserDir, sizeof(TheUserDir), "%s",AuthVpw.pw_dir);
+  snprintf( TheDomainDir, sizeof(TheDomainDir), "%s",
     vget_assign(TheDomain,NULL,0,&uid,&gid));
   snprintf(TheVpopmailDomains, sizeof(TheVpopmailDomains), "%s/domains", 
     VPOPMAILDIR);
@@ -1179,8 +1179,8 @@ int dom_info()
 {
  char *domain;
  domain_entry *entry;
- char *aliases[MAX_DOM_ALIAS];
- int  i, aliascount=0;
+ string_list aliases;
+ int i;
 
   if ( !(AuthVpw.pw_gid & SA_ADMIN) ) {
     show_error( ERR_NOT_AUTHORIZED, 1401 );
@@ -1209,9 +1209,11 @@ int dom_info()
   snprintf(WriteBuf,sizeof(WriteBuf), RET_OK_MORE);
   wait_write();
 
+  string_list_init(&aliases, 10);
+
   while( entry ) {
     if (strcmp(entry->domain, entry->realdomain) != 0) {
-      aliases[aliascount++] = strdup(entry->domain);
+      string_list_add(&aliases, entry->domain);
 
     } else {
       snprintf(WriteBuf,sizeof(WriteBuf),"domain %s" RET_CRLF, 
@@ -1235,14 +1237,13 @@ int dom_info()
     entry = get_domain_entries(NULL);
   }
 
-  for(i=0;i<aliascount;i++) {
+  for(i=0;i<aliases.count;i++) {
     snprintf(WriteBuf,sizeof(WriteBuf),"alias %s" RET_CRLF, 
-             aliases[i]);
+             aliases.values[i]);
     wait_write();
-    free( aliases[i] );
   } 
 
-
+  string_list_free(&aliases);
 
   snprintf(WriteBuf, sizeof(WriteBuf), "." RET_CRLF);
   return(0);
@@ -1279,7 +1280,7 @@ int validate_path(char *newpath, char *path)
 
   /* expand the path */
   if ( path[0] == '/' ) {
-    snprintf(newpath, MAXPATH, path);
+    snprintf(newpath, MAXPATH, "%s", path);
   } else { 
     slash = strchr( path, '/');
     if ( slash == NULL ) {
@@ -1325,7 +1326,7 @@ int validate_path(char *newpath, char *path)
           return(8);
         }
       }
-      snprintf(newpath, MAXPATH, myvpw->pw_dir);
+      snprintf(newpath, MAXPATH, "%s",myvpw->pw_dir);
       strncat(newpath, &path[i], MAXPATH );
     } else {     /*  may be domain name   */
       for(i=0;path[i]!='/'&&path[i]!=0&&i<256;++i) {
@@ -1336,7 +1337,7 @@ int validate_path(char *newpath, char *path)
         show_error( ERR_NOT_AUTH_DOMAIN, 1509 );
         return(9);
       } 
-      snprintf(newpath, MAXPATH, thedir);
+      snprintf(newpath, MAXPATH, "%s", thedir);
       strncat(newpath, &path[i], MAXPATH );
     }
   }
@@ -1388,7 +1389,7 @@ int mk_dir()
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s",RET_OK);
   return(0);
 }
 
@@ -1412,7 +1413,7 @@ int rm_dir()
       strerror(errno));
     return(-1);
   }
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK);
   return(0);
 }
 
@@ -1444,7 +1445,7 @@ int list_dir()
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
 
   while((mydirent=readdir(mydir))!=NULL){
@@ -1457,7 +1458,7 @@ int list_dir()
       printf("error on stat of %s\n", mydirent->d_name);
       exit(-1);
     }
-    snprintf( WriteBuf, sizeof(WriteBuf), mydirent->d_name);
+    snprintf( WriteBuf, sizeof(WriteBuf), "%s", mydirent->d_name);
     if ( S_ISREG(statbuf.st_mode ) ) {
       strncat(WriteBuf," file", sizeof(WriteBuf));
     } else if ( S_ISDIR(statbuf.st_mode ) ) {
@@ -1482,7 +1483,7 @@ int list_dir()
     /* oh well, at least we might die soon */
   }
 
-  snprintf(WriteBuf, sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf, sizeof(WriteBuf), "%s", "." RET_CRLF);
   return(0);
 }
 
@@ -1513,7 +1514,7 @@ int rename_file()
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK);
   return(0);
 
 }
@@ -1540,7 +1541,7 @@ int rm_file()
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK);
   return(0);
 }
 
@@ -1573,7 +1574,7 @@ int write_file()
   }
   fclose(fs);
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK);
   return(0);
 }
 
@@ -1598,12 +1599,12 @@ int read_file()
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
 
   while(fgets(tmpbuf,sizeof(tmpbuf),fs)!=NULL){
     if ( strcmp(tmpbuf, "." RET_CRLF) == 0 || strcmp(tmpbuf, ".\n") == 0 ) {
-      snprintf(WriteBuf, sizeof(WriteBuf), ".");
+      snprintf(WriteBuf, sizeof(WriteBuf), "%s", ".");
       strncat(WriteBuf, tmpbuf, sizeof(WriteBuf));
     } else {
       memcpy(WriteBuf,tmpbuf,sizeof(tmpbuf));
@@ -1613,9 +1614,9 @@ int read_file()
   fclose(fs);
 
   if ( tmpbuf[0] != 0 && tmpbuf[strlen(tmpbuf)-1] != '\n' ) {
-    snprintf(WriteBuf,sizeof(WriteBuf), RET_CRLF "." RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_CRLF "." RET_CRLF);
   } else {
-    snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
   }
   return(0);
 }
@@ -1640,7 +1641,7 @@ int stat_file()
       strerror(errno));
     return(-1);
   }
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
   snprintf(WriteBuf,sizeof(WriteBuf), "uid: %d\n", mystat.st_uid); 
   wait_write();
@@ -1687,7 +1688,7 @@ int list_domains()
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
 
   count = 0;
@@ -1709,7 +1710,7 @@ int list_domains()
     entry=get_domain_entries(NULL);
     
   }
-  snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
   return(0);
 }
 
@@ -1746,7 +1747,7 @@ int find_domain()
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
 
   count = 0;
@@ -1764,17 +1765,18 @@ int find_domain()
   }
 
   if( miss ) {
-    page = 0;
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
+    return(0);
   } else if( per_page > 0 ) {
     page = ( count / per_page ) + 1;
   } else {
-    page = count;
+    page = count + 1;
   }
 
   snprintf(WriteBuf,sizeof(WriteBuf), "page %i" RET_CRLF, page );
   wait_write();
 
-  snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
   return(0);
 }
 
@@ -1795,7 +1797,7 @@ int domain_count()
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
 
   count = 0;
@@ -1807,7 +1809,7 @@ int domain_count()
   snprintf(WriteBuf,sizeof(WriteBuf), "count %i" RET_CRLF, count);
   wait_write();
 
-  snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
   return(0);
 }
 
@@ -1840,7 +1842,7 @@ int user_count()
      return(-1);
    }
  
-   snprintf(WriteBuf, sizeof(WriteBuf), RET_OK_MORE);
+   snprintf(WriteBuf, sizeof(WriteBuf), "%s", RET_OK_MORE);
    wait_write();
  
    first=1;
@@ -1852,7 +1854,7 @@ int user_count()
 
    snprintf(WriteBuf,sizeof(WriteBuf), "count %i" RET_CRLF, count);
    wait_write();
-   snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+   snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
    return(0);
  }
   	 
@@ -1906,7 +1908,7 @@ int list_users()
     return(-1);
   }
 
-  snprintf(WriteBuf, sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf, sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
 
   first=1;
@@ -1923,13 +1925,13 @@ int list_users()
       send_user_info(tmpvpw);
     }
 
-    snprintf(WriteBuf, sizeof(WriteBuf), RET_CRLF );
+    snprintf(WriteBuf, sizeof(WriteBuf), "%s", RET_CRLF );
     wait_write();
 
     ++count;
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
   return(0);
 }
 
@@ -1984,7 +1986,7 @@ list_alias()
   /*  print all aliases  */
   if (strstr(Email, "@") == NULL) {
     tmpalias = valias_select_all(Alias, Email);
-    snprintf(WriteBuf, sizeof(WriteBuf), RET_OK_MORE);
+    snprintf(WriteBuf, sizeof(WriteBuf), "%s", RET_OK_MORE);
     wait_write();
       
     while (tmpalias != NULL) {
@@ -1997,9 +1999,9 @@ list_alias()
   } else {
     tmpalias = valias_select(Alias, Domain);
     if (tmpalias == NULL) {
-      snprintf(WriteBuf, sizeof(WriteBuf), RET_OK);
+      snprintf(WriteBuf, sizeof(WriteBuf), "%s", RET_OK);
     } else {
-      snprintf(WriteBuf, sizeof(WriteBuf), RET_OK_MORE);
+      snprintf(WriteBuf, sizeof(WriteBuf), "%s", RET_OK_MORE);
       wait_write();
       
       while (tmpalias != NULL) {
@@ -2010,7 +2012,7 @@ list_alias()
     }
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
   return(0);
 }
 
@@ -2073,7 +2075,7 @@ int list_lists()
     return(-1);
   }
 
-  snprintf(WriteBuf, sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf, sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
 
   j = bkscandir(".", &namelist, 0, qa_sort);
@@ -2102,7 +2104,7 @@ int list_lists()
     }
     ++count;
   }
-  snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
   return(0);
 }
 
@@ -2128,12 +2130,12 @@ int get_ip_map()
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf),RET_OK_MORE);
+  snprintf(WriteBuf,sizeof(WriteBuf),"%s", RET_OK_MORE);
   wait_write();
  
   snprintf(WriteBuf,sizeof(WriteBuf),"%s %s" RET_CRLF, ip, tmpdomain);
   wait_write();
-  snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
 
 #else
   show_error( ERR_NOT_AVAIL, 3004 );
@@ -2169,11 +2171,11 @@ int add_ip_map()
   }
 
   if ( vadd_ip_map(ip,domain) < 0 ) {
-    snprintf(WriteBuf,sizeof(WriteBuf), RET_ERR "0.3105 error" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_ERR "0.3105 error" RET_CRLF);
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf),RET_OK);
+  snprintf(WriteBuf,sizeof(WriteBuf),"%s",RET_OK);
 #else
   show_error( ERR_NOT_AVAIL, 3106 );
 #endif
@@ -2202,11 +2204,11 @@ int del_ip_map()
   }
 
   if ( vdel_ip_map(ip,domain) < 0 ) {
-    snprintf(WriteBuf,sizeof(WriteBuf), RET_ERR "0.3204 error" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_ERR "0.3204 error" RET_CRLF);
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf),RET_OK);
+  snprintf(WriteBuf,sizeof(WriteBuf),"%s",RET_OK);
 
 #else
   show_error( ERR_NOT_AVAIL, 3205 );
@@ -2226,7 +2228,7 @@ int show_ip_map()
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
 
   first = 1;
@@ -2236,7 +2238,7 @@ int show_ip_map()
     snprintf(WriteBuf, sizeof(WriteBuf), "%s %s" RET_CRLF, r_ip, r_domain);
     wait_write();
   }
-  snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
 
 #else
   show_error( ERR_NOT_AVAIL, 3302 );
@@ -2265,7 +2267,7 @@ int get_limits()
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
 
   snprintf(WriteBuf,sizeof(WriteBuf), "max_popaccounts %d" RET_CRLF, 
@@ -2278,74 +2280,74 @@ int get_limits()
     mylimits.maxautoresponders); wait_write();
   snprintf(WriteBuf,sizeof(WriteBuf), "max_mailinglists %d" RET_CRLF, 
     mylimits.maxmailinglists); wait_write();
-  snprintf(WriteBuf,sizeof(WriteBuf), "disk_quota %d" RET_CRLF, 
+  snprintf(WriteBuf,sizeof(WriteBuf), "disk_quota %llu" RET_CRLF, 
     mylimits.diskquota); wait_write();
-  snprintf(WriteBuf,sizeof(WriteBuf), "max_msgcount %d" RET_CRLF, 
+  snprintf(WriteBuf,sizeof(WriteBuf), "max_msgcount %llu" RET_CRLF, 
     mylimits.maxmsgcount); wait_write();
-  snprintf(WriteBuf,sizeof(WriteBuf), "default_quota %d" RET_CRLF, 
+  snprintf(WriteBuf,sizeof(WriteBuf), "default_quota %llu" RET_CRLF, 
     mylimits.defaultquota); wait_write();
-  snprintf(WriteBuf,sizeof(WriteBuf), "default_maxmsgcount %d" RET_CRLF, 
+  snprintf(WriteBuf,sizeof(WriteBuf), "default_maxmsgcount %llu" RET_CRLF, 
     mylimits.defaultmaxmsgcount); wait_write();
 
   if (mylimits.disable_pop) 
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_pop 1" RET_CRLF); 
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_pop 1" RET_CRLF); 
   else 
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_pop 0" RET_CRLF); 
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_pop 0" RET_CRLF); 
 
   wait_write();
 
   if (mylimits.disable_imap) 
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_imap 1" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_imap 1" RET_CRLF);
   else
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_imap 0" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_imap 0" RET_CRLF);
   wait_write();
 
   if (mylimits.disable_dialup) 
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_dialup 1" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_dialup 1" RET_CRLF);
   else 
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_dialup 0" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_dialup 0" RET_CRLF);
   wait_write();
 
   if (mylimits.disable_passwordchanging) 
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_password_changing 1" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_password_changing 1" RET_CRLF);
   else 
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_password_changing 0" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_password_changing 0" RET_CRLF);
   wait_write();
 
   if (mylimits.disable_webmail)
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_webmail 1" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_webmail 1" RET_CRLF);
   else
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_webmail 0" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_webmail 0" RET_CRLF);
   wait_write();
 
   if (mylimits.disable_relay)
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_external_relay 1" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_external_relay 1" RET_CRLF);
   else
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_external_relay 0" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_external_relay 0" RET_CRLF);
   wait_write();
 
   if (mylimits.disable_smtp)
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_smtp 1" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_smtp 1" RET_CRLF);
   else
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_smtp 0" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_smtp 0" RET_CRLF);
   wait_write();
 
   if (mylimits.disable_spamassassin) 
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_spamassassin 1" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_spamassassin 1" RET_CRLF);
   else 
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_spamassassin 0" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_spamassassin 0" RET_CRLF);
   wait_write();
 
   if (mylimits.delete_spam)
-    snprintf(WriteBuf,sizeof(WriteBuf), "delete_spam 1" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "delete_spam 1" RET_CRLF);
   else
-    snprintf(WriteBuf,sizeof(WriteBuf), "delete_spam 0" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "delete_spam 0" RET_CRLF);
   wait_write();
 
   if (mylimits.disable_maildrop) 
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_maildrop 1" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_maildrop 1" RET_CRLF);
   else 
-    snprintf(WriteBuf,sizeof(WriteBuf), "disable_maildrop 0" RET_CRLF);
+    snprintf(WriteBuf,sizeof(WriteBuf), "%s", "disable_maildrop 0" RET_CRLF);
   wait_write();
 
   snprintf(WriteBuf,sizeof(WriteBuf), "perm_account %d" RET_CRLF, 
@@ -2368,7 +2370,7 @@ int get_limits()
     mylimits.perm_defaultquota); wait_write();
   wait_write();
 
-  snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
   return(0);
 }
 
@@ -2390,14 +2392,14 @@ int set_limits()
     return(-1);
   }
 
-  snprintf(domain,sizeof(domain), param);
+  snprintf(domain,sizeof(domain), "%s", param);
 
   if ((ret=vget_limits(domain,&mylimits))!=0){
     show_error( ERR_NO_GET_LIMITS, 3505 );
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK);
   wait_write();
 
   while(fgets(ReadBuf,sizeof(ReadBuf),stdin)!=NULL ) {
@@ -2405,7 +2407,7 @@ int set_limits()
 
     if ( (param = strtok(ReadBuf,PARAM_TOKENS)) == NULL ) continue;
     if ( (value = strtok(NULL,PARAM_TOKENS)) == NULL )  {
-      snprintf(WriteBuf,sizeof(WriteBuf), 
+      snprintf(WriteBuf,sizeof(WriteBuf), "%s",
         "  Warning: missing value" RET_CRLF);
       wait_write();
       continue;
@@ -2468,7 +2470,7 @@ int set_limits()
     } else if ( strcmp(param,"perm_defaultquota") == 0 ) {
       mylimits.perm_defaultquota = atoi(value);
     } else {
-      snprintf(WriteBuf,sizeof(WriteBuf), 
+      snprintf(WriteBuf,sizeof(WriteBuf), "%s", 
         "  Warning: invalid option" RET_CRLF);
       wait_write();
     }
@@ -2478,7 +2480,7 @@ int set_limits()
   if ( vset_limits(domain,&mylimits) < 0 ) {
     show_error( ERR_NO_SET_LIMITS, 3504 );
   } else {
-    snprintf(WriteBuf,sizeof(WriteBuf),RET_OK);
+    snprintf(WriteBuf,sizeof(WriteBuf),"%s",RET_OK);
   }
   return(0);
 }
@@ -2504,7 +2506,7 @@ int del_limits()
     return(-1);
   }
 
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK);
   return(0);
 }
 
@@ -2539,9 +2541,9 @@ int get_lastauth()
   } 
 
 #ifndef ENABLE_AUTH_LOGGING
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK);
 #else
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
 
   snprintf(WriteBuf, sizeof(WriteBuf), "time %ld" RET_CRLF,
@@ -2552,7 +2554,7 @@ int get_lastauth()
     vget_lastauthip(tmpvpw, TmpDomain));
   wait_write();
 
-  snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
 #endif
 
   return(0);
@@ -2579,35 +2581,36 @@ int mod_list()
 int get_user_size()
 {
   char *email_address;
-  int ret, cnt;
-  long bytes;
+  int ret;
+  storage_t cnt;
+  storage_t bytes;
 
   if (!(AuthVpw.pw_gid & QA_ADMIN) && !(AuthVpw.pw_gid & SA_ADMIN)) {
-    snprintf(WriteBuf, sizeof(WriteBuf), RET_ERR "3801 not authorized" RET_CRLF);
+    snprintf(WriteBuf, sizeof(WriteBuf), "%s", RET_ERR "3801 not authorized" RET_CRLF);
     return(-1);
   }
 
   if ((email_address = strtok(NULL, TOKENS)) == NULL) {
-    snprintf(WriteBuf, sizeof(WriteBuf), 
+    snprintf(WriteBuf, sizeof(WriteBuf), "%s",
       RET_ERR "3802 email_address required" RET_CRLF);
     return(-1);
   }
 
   if (parse_email(email_address, TmpUser, TmpDomain, AUTH_SIZE) != 0) {
-    snprintf(WriteBuf, sizeof(WriteBuf), 
+    snprintf(WriteBuf, sizeof(WriteBuf), "%s",
       RET_ERR "3803 invalid email address" RET_CRLF);
     return(-1);
   }
 
   if (!(AuthVpw.pw_gid & SA_ADMIN) && (AuthVpw.pw_gid & QA_ADMIN) 
     && (strcmp(TheDomain, TmpDomain) != 0)) {
-    snprintf(WriteBuf, sizeof(WriteBuf), 
+    snprintf(WriteBuf, sizeof(WriteBuf), "%s",
       RET_ERR "3804 not authorized for domain" RET_CRLF);
     return(-1);
   }
 
   if ((tmpvpw = vauth_getpw(TmpUser, TmpDomain)) == NULL) {
-    snprintf(WriteBuf, sizeof(WriteBuf), 
+    snprintf(WriteBuf, sizeof(WriteBuf), "%s",
       RET_ERR "3805 user does not exist" RET_CRLF);
     return(-1);
   }
@@ -2615,18 +2618,18 @@ int get_user_size()
   bytes = 0;
   cnt = 0;
   if ((ret = readuserquota(tmpvpw->pw_dir, &bytes, &cnt)) != 0) {
-    snprintf(WriteBuf, sizeof(WriteBuf), 
+    snprintf(WriteBuf, sizeof(WriteBuf), "%s",
       RET_ERR "3806 unable to fetch size of user account" RET_CRLF);
     return(-1);
   }
 
-  snprintf(WriteBuf, sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf, sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
-  snprintf(WriteBuf, sizeof(WriteBuf), "size %ld" RET_CRLF, bytes);
+  snprintf(WriteBuf, sizeof(WriteBuf), "size %llu" RET_CRLF, bytes);
   wait_write();
-  snprintf(WriteBuf, sizeof(WriteBuf), "count %d" RET_CRLF, cnt);
+  snprintf(WriteBuf, sizeof(WriteBuf), "count %llu" RET_CRLF, cnt);
   wait_write();
-  snprintf(WriteBuf, sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf, sizeof(WriteBuf), "%s", "." RET_CRLF);
 
   return(0);
 }
@@ -2634,31 +2637,32 @@ int get_user_size()
 int get_domain_size()
 {
   char *domain, tmpdir[256];
-  int ret, cnt, first;
-  long bytes;
-  unsigned int totalcnt;
-  unsigned long long totalbytes;
+  int ret, first;
+  storage_t cnt;
+  storage_t bytes;
+  storage_t totalcnt;
+  storage_t totalbytes;
 
   if (!(AuthVpw.pw_gid & QA_ADMIN) && !(AuthVpw.pw_gid & SA_ADMIN)) {
-    snprintf(WriteBuf, sizeof(WriteBuf), 
+    snprintf(WriteBuf, sizeof(WriteBuf), "%s",
       RET_ERR "3901 not authorized" RET_CRLF);
     return(-1);
   }
 
   if ((domain = strtok(NULL, TOKENS)) == NULL) {
-    snprintf(WriteBuf, sizeof(WriteBuf), 
+    snprintf(WriteBuf, sizeof(WriteBuf), "%s",
       RET_ERR "3902 domain required" RET_CRLF);
     return(-1);
   }
 
   if (!(AuthVpw.pw_gid & SA_ADMIN) && (AuthVpw.pw_gid & QA_ADMIN) 
     && (strcmp(TheDomain, domain) != 0)) {
-    snprintf(WriteBuf, sizeof(WriteBuf),
+    snprintf(WriteBuf, sizeof(WriteBuf), "%s",
       RET_ERR "3903 not authorized for domain" RET_CRLF);
     return(-1);
   }
 
-  snprintf(WriteBuf, sizeof(WriteBuf), RET_OK_MORE);
+  snprintf(WriteBuf, sizeof(WriteBuf), "%s", RET_OK_MORE);
   wait_write();
 
   totalbytes = 0;
@@ -2676,9 +2680,9 @@ int get_domain_size()
     } else {
       snprintf(WriteBuf, sizeof(WriteBuf), "user %s@%s" RET_CRLF, tmpvpw->pw_name, domain);
       wait_write();
-      snprintf(WriteBuf, sizeof(WriteBuf), "size %ld" RET_CRLF, bytes);
+      snprintf(WriteBuf, sizeof(WriteBuf), "size %llu" RET_CRLF, bytes);
       wait_write();
-      snprintf(WriteBuf, sizeof(WriteBuf), "count %d" RET_CRLF, cnt);
+      snprintf(WriteBuf, sizeof(WriteBuf), "count %llu" RET_CRLF, cnt);
       wait_write();
       totalbytes += (unsigned long)bytes;
       totalcnt += (unsigned int)cnt;
@@ -2689,16 +2693,16 @@ int get_domain_size()
   wait_write();
   snprintf(WriteBuf, sizeof(WriteBuf), "size %llu" RET_CRLF, totalbytes);
   wait_write();
-  snprintf(WriteBuf, sizeof(WriteBuf), "count %u" RET_CRLF, totalcnt);
+  snprintf(WriteBuf, sizeof(WriteBuf), "count %llu" RET_CRLF, totalcnt);
   wait_write();
-  snprintf(WriteBuf, sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf, sizeof(WriteBuf), "%s", "." RET_CRLF);
 
   return(0);
 }
 
 int quit()
 {
-  snprintf(WriteBuf,sizeof(WriteBuf), RET_OK);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", RET_OK);
   wait_write();
   vclose();
 
@@ -2724,7 +2728,7 @@ int login_help()
     }
     wait_write();
   }
-  snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
   
   return(1); 
 }
@@ -2750,7 +2754,7 @@ int help()
       wait_write();
     }
   }
-  snprintf(WriteBuf,sizeof(WriteBuf), "." RET_CRLF);
+  snprintf(WriteBuf,sizeof(WriteBuf), "%s", "." RET_CRLF);
 
   return(1); 
 }
